@@ -47,6 +47,7 @@ class Truss():
         self.name = name
         self.graph = plt.figure(name)
         self.view = self.graph.add_subplot(111,projection='3d')
+
         plt.show(block=False)
         plt.ion()
 
@@ -162,25 +163,25 @@ class Truss():
             r2_end = unpack('!f',pack('4b',self.db0[98],self.db0[99],self.db0[100],self.db0[101]))[0]
 
         async def x(self,start,end):
-            for interval in np.linspace(start,end,int(abs(start - end) / 100)):
+            for interval in np.linspace(start,end,int(abs(start - end) / 100)+1):
                 if index == 0: self.db1[2],self.db1[3],self.db1[4],self.db1[5] = pack('!f',interval)
                 elif index == 1: self.db1[18],self.db1[19],self.db1[20],self.db1[21] = pack('!f',interval)
                 self.x[index] = interval
                 await asyncio.sleep(0.01)
         async def y(self,start,end):
-            for interval in np.linspace(start,end,int(abs(start - end) / 100)):
+            for interval in np.linspace(start,end,int(abs(start - end) / 100)+1):
                 if index == 0: self.db1[6],self.db1[7],self.db1[8],self.db1[9] = pack('!f',interval)
                 elif index == 1: self.db1[22],self.db1[23],self.db1[24],self.db1[25] = pack('!f',interval)
                 self.y[index] = interval
                 await asyncio.sleep(0.01)
         async def z(self,start,end):
-            for interval in np.linspace(start,end,int(abs(start - end) / 40)):
+            for interval in np.linspace(start,end,int(abs(start - end) / 40)+1):
                 if index == 0: self.db1[10],self.db1[11],self.db1[12],self.db1[13] = pack('!f',interval)
                 elif index == 1: self.db1[26],self.db1[27],self.db1[28],self.db1[29] = pack('!f',interval)
                 self.z[index] = interval
                 await asyncio.sleep(0.001)
         async def r(self,start,end):
-            for interval in np.linspace(start,end,int(abs(start - end) / 10)):
+            for interval in np.linspace(start,end,int(abs(start - end) / 10)+1):
                 if index == 0: self.db1[14],self.db1[15],self.db1[16],self.db1[17] = pack('!f',interval)
                 elif index == 1: self.db1[30],self.db1[31],self.db1[32],self.db1[33] = pack('!f',interval)
                 self.r[index] = interval
@@ -283,18 +284,20 @@ class Truss():
 
             self.status = dson(self.status_file)
             self.params = dson(self.params_file)
+            if 'plate_name' not in self.status:
+                continue
             plate_name = self.status['plate_name']
             self.plate = dson(f'{self.plate_dir}/{plate_name}.json')
 
         self.view.cla()
 
         #区域
-        self.view.text(self.area_x[1],0,0,'X')
-        self.view.text(0,self.area_y[1],0,'Y')
-        self.view.text(0,0,self.area_z[1],'Z')
-        self.view.plot([self.area_x[0]-750,self.area_x[1]+750],[0,0],[0,0])
-        self.view.plot([0,0],self.area_y,[0,0])
-        self.view.plot([0,0],[0,0],[self.area_z[0],self.area_z[1]])
+        self.view.text(self.area_x[1],0,0,'+X')
+        self.view.text(0,self.area_y[1],0,'+Y')
+        self.view.text(0,0,self.area_z[0],'-Z')
+        self.view.plot([self.area_x[0]-750,self.area_x[1]+750],[0,0],[0,0],linewidth=1,linestyle='--')
+        self.view.plot([0,0],self.area_y,[0,0],linewidth=1,linestyle='--')
+        self.view.plot([0,0],[0,0],[self.area_z[0],self.area_z[1]],linewidth=1,linestyle='--')
 
         try:
             #画钢板
@@ -312,7 +315,8 @@ class Truss():
             plate_p3 = rotate(plate_corner,(self.plate_left,self.plate_bottom,self.grab_z),plate_degree)
             
             self.rectangle(plate_p0,plate_p1,plate_p2,plate_p3)
-            self.point(plate_p1,f'{plate_p1} {plate_degree}°')
+            self.view.scatter(plate_p1[0],plate_p1[1],plate_p1[2])
+            self.view.text(plate_p1[0],plate_p1[1],plate_p1[2],f'{plate_p1} {plate_degree}°')
 
             #画托盘
             drop_pos = self.params['sorting.pallet_pos']
@@ -323,7 +327,7 @@ class Truss():
             p2 = drop_pos[0] + pallet_size[0], drop_pos[1] + pallet_size[1], drop_pos[2]
             p3 = drop_pos[0] + pallet_size[0], drop_pos[1], drop_pos[2]
             self.rectangle(p0,p1,p2,p3)
-            self.text(p2,str(drop_pos))
+            self.view.text(p3[0],p3[1],p3[2],s=str(drop_pos))
 
             if self.plate['finished']:
                 drop_positions = self.plate['tiling'][-1]
@@ -332,6 +336,8 @@ class Truss():
                 drop_positions = self.plate['tiling'][tiling_index]
 
             for part in self.plate['parts']:
+                from copy import deepcopy
+                part = deepcopy(part)
                 id = part['id']
                 handle = part['handle']
                 center = part['center']
@@ -392,10 +398,11 @@ class Truss():
 
                 if 0 == handle:
                     a_grab_pos = rotate(plate_corner,a_grab_pos,plate_degree)
-                    
+                    self.view.scatter(a_grab_pos[0],a_grab_pos[1],a_grab_pos[2],s=3)
                     drop_x = drop_pos[0] + a_drop_pos_offset[0]
                     drop_y = drop_pos[1] + a_drop_pos_offset[1]
                     drop_z = drop_pos[2] + part_size[2]
+                    self.view.scatter(drop_x,drop_y,drop_z,s=3)
                     
                     box_left = drop_pos[0] - part_size[0] / 2  
                     box_top = drop_pos[1] - part_size[1] / 2
@@ -408,9 +415,11 @@ class Truss():
                     self.rectangle(p0, p1, p2, p3)
                 elif 1 == handle:
                     b_grab_pos = rotate(plate_corner,b_grab_pos,plate_degree)
+                    self.view.scatter(b_grab_pos[0],b_grab_pos[1],b_grab_pos[2],s=3)
                     drop_x = drop_pos[0] + b_drop_pos_offset[0]
                     drop_y = drop_pos[1] + b_drop_pos_offset[1]
                     drop_z = drop_pos[2] + part_size[2]
+                    self.view.scatter(drop_x,drop_y,drop_z,s=3)
 
                     box_left = drop_pos[0] - part_size[0] / 2  
                     box_top = drop_pos[1] - part_size[1] / 2
@@ -423,13 +432,18 @@ class Truss():
                     self.rectangle(p0, p1, p2, p3)
                 else:
                     a_grab_pos = rotate(plate_corner,a_grab_pos,plate_degree)
+                    self.view.scatter(a_grab_pos[0],a_grab_pos[1],a_grab_pos[2],s=3)
                     a_drop_x = drop_pos[0] + a_drop_pos_offset[0]
                     a_drop_y = drop_pos[1] + a_drop_pos_offset[1]
                     a_drop_z = drop_pos[2]
+                    self.view.scatter(a_drop_x,a_drop_y,a_drop_z,s=3)
+
                     b_grab_pos = rotate(plate_corner,b_grab_pos,plate_degree)
+                    self.view.scatter(b_grab_pos[0],b_grab_pos[1],b_grab_pos[2],s=3)
                     b_drop_x = drop_pos[0] + b_drop_pos_offset[0]
                     b_drop_y = drop_pos[1] + b_drop_pos_offset[1]
                     b_drop_z = drop_pos[2]
+                    self.view.scatter(b_drop_x,b_drop_y,b_drop_z,s=3)
 
                     box_left = drop_pos[0] - part_size[0] / 2  
                     box_top = drop_pos[1] - part_size[1] / 2
@@ -444,20 +458,20 @@ class Truss():
         except:pass
                     
         #桁架
-        self.view.plot([self.x[0],self.x[0]],[self.area_y[0],self.area_y[1]],[self.area_z[1],self.area_z[1]],c='red')
-        self.view.plot([self.x[1],self.x[1]],[self.area_y[0],self.area_y[1]],[self.area_z[1],self.area_z[1]],c='red')
-        self.view.plot([self.x[0],self.x[0]],[self.y[0],self.y[0]],[self.z[0],self.area_z[1]],c='red')
-        self.view.plot([self.x[1],self.x[1]],[self.y[1],self.y[1]],[self.z[1],self.area_z[1]],c='red')
+        self.view.plot([self.x[0],self.x[0]],[self.area_y[0],self.area_y[1]],[self.area_z[1],self.area_z[1]],c='red',linewidth=1)
+        self.view.plot([self.x[1],self.x[1]],[self.area_y[0],self.area_y[1]],[self.area_z[1],self.area_z[1]],c='red',linewidth=1)
+        self.view.plot([self.x[0],self.x[0]],[self.y[0],self.y[0]],[self.z[0],self.area_z[1]],c='red',linewidth=1)
+        self.view.plot([self.x[1],self.x[1]],[self.y[1],self.y[1]],[self.z[1],self.area_z[1]],c='red',linewidth=1)
         
         #抓手A
         x0,y0,_=rotate([self.x[0],self.y[0],0],[self.x[0]-750,self.y[0],0],-self.r[0])
         x1,y1,_=rotate([self.x[0],self.y[0],0],[self.x[0]+750,self.y[0],0],-self.r[0])
-        self.view.plot([x0,x1],[y0,y1],[self.z[0],self.z[0]],c='blue')
+        self.view.plot([x0,x1],[y0,y1],[self.z[0],self.z[0]],c='blue',linewidth=1)
 
         #抓手B
         x0,y0,_=rotate([self.x[1],self.y[1],0],[self.x[1]-750,self.y[1],0],-self.r[1])
         x1,y1,_=rotate([self.x[1],self.y[1],0],[self.x[1]+750,self.y[1],0],-self.r[1])
-        self.view.plot([x0,x1],[y0,y1],[self.z[1],self.z[1]],c='blue')
+        self.view.plot([x0,x1],[y0,y1],[self.z[1],self.z[1]],c='blue',linewidth=1)
 
         #位置
         self.view.text(self.x[0],self.y[0],self.z[0],str([int(self.x[0]),int(self.y[0]),int(self.z[0]),int(self.r[0])]))
@@ -468,18 +482,12 @@ class Truss():
         await asyncio.sleep(0.001)
         
         self.loop.create_task(self.update())
-    def text(self,p,s=''):
-        self.view.text(p[0],p[1],p[2],s=s)
-    def point(self,p,s=''):
-        if p:
-            self.view.text(p[0],p[1],p[2],s=s)
-            self.view.scatter(p[0],p[1],p[2])
     def rectangle(self,p0,p1,p2,p3,c=None,s=''):
         #可视化：钢板
         self.view.plot(
             [p0[0],p1[0],p2[0],p3[0],p0[0]],
             [p0[1],p1[1],p2[1],p3[1],p0[1]],
-            [p0[2],p1[2],p2[2],p3[2],p0[2]])
+            [p0[2],p1[2],p2[2],p3[2],p0[2]],linewidth=.5)
         if c: self.view.text(c[0],c[1],c[2],s=s)
     def show(self):
         self.loop.create_task(self.update())
